﻿/*
 * CKEditor Html Editor Provider for DotNetNuke
 * ========
 * http://dnnckeditor.codeplex.com/
 * Copyright (C) Ingo Herbote
 *
 * The software, this file and its contents are subject to the CKEditor Provider
 * License. Please read the license.txt file before using, installing, copying,
 * modifying or distribute this file or part of its contents. The contents of
 * this file is part of the Source Code of the CKEditor Provider.
 */

namespace WatchersNET.CKEditor.Controls
{
    using System;
    using System.Collections;
    using System.IO;
    using System.Linq;
    using System.Web.UI.HtmlControls;
    using System.Web.UI.WebControls;

    using DotNetNuke.Common.Utilities;
    using DotNetNuke.Entities.Portals;
    using DotNetNuke.Security;
    using DotNetNuke.Security.Permissions;
    using DotNetNuke.Services.Exceptions;
    using DotNetNuke.Services.FileSystem;
    using DotNetNuke.Services.Localization;
    using DotNetNuke.UI.Utilities;

    using Microsoft.VisualBasic;

    using FileInfo = DotNetNuke.Services.FileSystem.FileInfo;

    /// <summary>
    /// The url control.
    /// </summary>
    public abstract class UrlControl : DotNetNuke.Framework.UserControlBase
    {
        /// <summary>
        /// The _do change url.
        /// </summary>
        private bool _doChangeURL;

        /// <summary>
        /// The _do reload files.
        /// </summary>
        private bool _doReloadFiles;

        /// <summary>
        /// The _do reload folders.
        /// </summary>
        private bool _doReloadFolders;

        /// <summary>
        /// The _do render type controls.
        /// </summary>
        private bool _doRenderTypeControls;

        /// <summary>
        /// The _do render types.
        /// </summary>
        private bool _doRenderTypes;

        /// <summary>
        /// The _local resource file.
        /// </summary>
        private string _localResourceFile;

        /// <summary>
        /// The _portal info
        /// </summary>
        private PortalInfo _portalInfo;

        /// <summary>
        /// The with events field folders.
        /// </summary>
        private DropDownList folders;

        /// <summary>
        /// The with events field add.
        /// </summary>
        private LinkButton add;

        /// <summary>
        /// The with events field cancel.
        /// </summary>
        private LinkButton cancel;

        /// <summary>
        /// The with events field delete.
        /// </summary>
        private LinkButton delete;

        /// <summary>
        /// The with events field save.
        /// </summary>
        private LinkButton save;

        /// <summary>
        /// The with events field select.
        /// </summary>
        private LinkButton select;

        /// <summary>
        /// The with events field upload.
        /// </summary>
        private LinkButton upload;

        /// <summary>
        /// The with events field type.
        /// </summary>
        private RadioButtonList type;

        #region Constructors and Destructors

        /// <summary>
        /// Initializes a new instance of the <see cref="UrlControl"/> class.
        /// </summary>
        protected UrlControl()
        {
            this.PreRender += this.Page_PreRender;
            this.Load += this.Page_Load;
            this.Init += this.Page_Init;
        }

        #endregion

        #region Public Properties

        /// <summary>
        /// Gets or sets the file filter.
        /// </summary>
        public string FileFilter
        {
            get
            {
                return this.ViewState["FileFilter"] != null
                           ? Convert.ToString(this.ViewState["FileFilter"])
                           : string.Empty;
            }

            set
            {
                this.ViewState["FileFilter"] = value;
                if (this.IsTrackingViewState)
                {
                    this._doReloadFiles = true;
                }
            }
        }

        /// <summary>
        /// Gets or sets the local resource file.
        /// </summary>
        public string LocalResourceFile
        {
            get
            {
                return string.IsNullOrEmpty(this._localResourceFile)
                           ? string.Format(
                               "{0}/{1}/URLControl.ascx.resx",
                               this.TemplateSourceDirectory.Replace(
                                   "Providers/HtmlEditorProviders/CKEditor", "controls"),
                               Localization.LocalResourceDirectory)
                           : this._localResourceFile;
            }

            set
            {
                this._localResourceFile = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether show files.
        /// </summary>
        public bool ShowFiles
        {
            get
            {
                return this.ViewState["ShowFiles"] == null || Convert.ToBoolean(this.ViewState["ShowFiles"]);
            }

            set
            {
                this.ViewState["ShowFiles"] = value;
                if (this.IsTrackingViewState)
                {
                    this._doRenderTypes = true;
                }
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether show secure.
        /// </summary>
        public bool ShowSecure
        {
            get
            {
                return this.ViewState["ShowSecure"] == null || Convert.ToBoolean(this.ViewState["ShowSecure"]);
            }

            set
            {
                this.ViewState["ShowSecure"] = value;
                if (this.IsTrackingViewState)
                {
                    this._doReloadFolders = true;
                }
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether show up load.
        /// </summary>
        public bool ShowUpLoad
        {
            get
            {
                return this.ViewState["ShowUpLoad"] == null || Convert.ToBoolean(this.ViewState["ShowUpLoad"]);
            }

            set
            {
                this.ViewState["ShowUpLoad"] = value;
                if (this.IsTrackingViewState)
                {
                    this._doRenderTypeControls = true;
                }
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether show URLs.
        /// </summary>
        public bool ShowUrls
        {
            get
            {
                return this.ViewState["ShowUrls"] == null || Convert.ToBoolean(this.ViewState["ShowUrls"]);
            }

            set
            {
                this.ViewState["ShowUrls"] = value;
                if (this.IsTrackingViewState)
                {
                    this._doRenderTypes = true;
                }
            }
        }

        /// <summary>
        /// Gets or sets the url.
        /// </summary>
        public string Url
        {
            get
            {
                string r = string.Empty;
                string strCurrentType = string.Empty;
                if (this.Type.Items.Count > 0 && this.Type.SelectedIndex >= 0)
                {
                    strCurrentType = this.Type.SelectedItem.Value;
                }

                switch (strCurrentType)
                {
                    case "U":
                        if (this.Urls.Visible)
                        {
                            if (this.Urls.SelectedItem != null)
                            {
                                r = this.Urls.SelectedItem.Value;
                                this.UrlTextBox.Text = r;
                            }
                        }
                        else
                        {
                            string mCustomUrl = this.UrlTextBox.Text;
                            r = mCustomUrl.ToLower() == "http://"
                                    ? string.Empty
                                    : DotNetNuke.Common.Globals.AddHTTP(mCustomUrl);
                        }

                        break;
                    case "F":
                        if (this.Files.SelectedItem != null)
                        {
                            if (!string.IsNullOrEmpty(this.Files.SelectedItem.Value))
                            {
                                r = "FileID=" + this.Files.SelectedItem.Value;
                            }
                            else
                            {
                                r = string.Empty;
                            }
                        }

                        break;
                }

                return r;
            }

            set
            {
                this.ViewState["Url"] = value;
                if (this.IsTrackingViewState)
                {
                    this._doChangeURL = true;
                }
            }
        }

        /// <summary>
        /// Gets or sets the url type.
        /// </summary>
        public string UrlType
        {
            get
            {
                return Convert.ToString(this.ViewState["UrlType"]);
            }

            set
            {
                if ((value == null) || string.IsNullOrEmpty(Strings.Trim(value)))
                {
                    return;
                }

                this.ViewState["UrlType"] = value;
                if (this.IsTrackingViewState)
                {
                    this._doChangeURL = true;
                }
            }
        }

        /// <summary>
        /// Gets or sets the width.
        /// </summary>
        public string Width
        {
            get
            {
                return Convert.ToString(this.ViewState["SkinControlWidth"]);
            }

            set
            {
                if (string.IsNullOrEmpty(value))
                {
                    return;
                }

                this.Urls.Width = Unit.Parse(value);
                this.UrlTextBox.Width = Unit.Parse(value);
                this.Folders.Width = Unit.Parse(value);
                this.Files.Width = Unit.Parse(value);
                this.ViewState["SkinControlWidth"] = value;
            }
        }

        #endregion

        #region Fields

        /// <summary>
        /// Gets or sets the type row
        /// </summary>
        /// <value>
        /// The type row.
        /// </value>
        protected HtmlTableRow TypeRow { get; set; }

        /// <summary>
        /// Gets or sets the file row.
        /// </summary>
        /// <value>
        /// The file row.
        /// </value>
        protected HtmlTableRow FileRow { get; set; }

        /// <summary>
        /// Gets or sets the url row.
        /// </summary>
        /// <value>
        /// The URL row.
        /// </value>
        protected HtmlTableRow URLRow { get; set; }

        /// <summary>
        /// Gets or sets the files.
        /// </summary>
        /// <value>
        /// The files.
        /// </value>
        protected DropDownList Files { get; set; }

        /// <summary>
        /// Gets or sets the URLs.
        /// </summary>
        /// <value>
        /// The URLs.
        /// </value>
        protected DropDownList Urls { get; set; }

        /// <summary>
        /// Gets or sets the storage location type image.
        /// </summary>
        /// <value>
        /// The storage location type image.
        /// </value>
        protected Image StorageLocationTypeImage { get; set; }

        /// <summary>
        /// Gets or sets the file label.
        /// </summary>
        /// <value>
        /// The file label.
        /// </value>
        protected Label FileLabel { get; set; }

        /// <summary>
        /// Gets or sets the folder label.
        /// </summary>
        /// <value>
        /// The folder label.
        /// </value>
        protected Label FolderLabel { get; set; }

        /// <summary>
        /// Gets or sets the message label.
        /// </summary>
        /// <value>
        /// The message label.
        /// </value>
        protected Label MessageLabel { get; set; }

        /// <summary>
        /// Gets or sets the URL label.
        /// </summary>
        /// <value>
        /// The URL label.
        /// </value>
        protected Label URLLabel { get; set; }

        /// <summary>
        /// Gets or sets the URL type label.
        /// </summary>
        /// <value>
        /// The URL type label.
        /// </value>
        protected Label URLTypeLabel { get; set; }

        /// <summary>
        /// Gets or sets the file.
        /// </summary>
        /// <value>
        /// The file.
        /// </value>
        protected HtmlInputFile File { get; set; }

        /// <summary>
        /// Gets or sets the URL text box.
        /// </summary>
        /// <value>
        /// The URL text box.
        /// </value>
        protected TextBox UrlTextBox { get; set; }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the folders.
        /// </summary>
        protected DropDownList Folders
        {
            get
            {
                return this.folders;
            }

            set
            {
                if (this.folders != null)
                {
                    this.folders.SelectedIndexChanged -= this.Folders_SelectedIndexChanged;
                }

                this.folders = value;
                if (this.folders != null)
                {
                    this.folders.SelectedIndexChanged += this.Folders_SelectedIndexChanged;
                }
            }
        }

        /// <summary>
        /// Gets or sets the add.
        /// </summary>
        protected LinkButton Add
        {
            get
            {
                return this.add;
            }

            set
            {
                if (this.add != null)
                {
                    this.add.Click -= this.Add_Click;
                }

                this.add = value;
                if (this.add != null)
                {
                    this.add.Click += this.Add_Click;
                }
            }
        }

        /// <summary>
        /// Gets or sets the cancel.
        /// </summary>
        protected LinkButton Cancel
        {
            get
            {
                return this.cancel;
            }

            set
            {
                if (this.cancel != null)
                {
                    this.cancel.Click -= this.Cancel_Click;
                }

                this.cancel = value;
                if (this.cancel != null)
                {
                    this.cancel.Click += this.Cancel_Click;
                }
            }
        }

        /// <summary>
        /// Gets or sets the delete.
        /// </summary>
        protected LinkButton Delete
        {
            get
            {
                return this.delete;
            }

            set
            {
                if (this.delete != null)
                {
                    this.delete.Click -= this.Delete_Click;
                }

                this.delete = value;
                if (this.delete != null)
                {
                    this.delete.Click += this.Delete_Click;
                }
            }
        }

        /// <summary>
        /// Gets or sets the save.
        /// </summary>
        protected LinkButton Save
        {
            get
            {
                return this.save;
            }

            set
            {
                if (this.save != null)
                {
                    this.save.Click -= this.Save_Click;
                }

                this.save = value;
                if (this.save != null)
                {
                    this.save.Click += this.Save_Click;
                }
            }
        }

        /// <summary>
        /// Gets or sets the select.
        /// </summary>
        protected LinkButton Select
        {
            get
            {
                return this.select;
            }

            set
            {
                if (this.select != null)
                {
                    this.select.Click -= this.Select_Click;
                }

                this.select = value;
                if (this.select != null)
                {
                    this.select.Click += this.Select_Click;
                }
            }
        }

        /// <summary>
        /// Gets or sets the upload.
        /// </summary>
        protected LinkButton Upload
        {
            get
            {
                return this.upload;
            }

            set
            {
                if (this.upload != null)
                {
                    this.upload.Click -= this.Upload_Click;
                }

                this.upload = value;
                if (this.upload != null)
                {
                    this.upload.Click += this.Upload_Click;
                }
            }
        }

        /// <summary>
        /// Gets or sets the type list.
        /// </summary>
        protected RadioButtonList Type
        {
            get
            {
                return this.type;
            }

            set
            {
                if (this.type != null)
                {
                    this.type.SelectedIndexChanged -= this.Type_SelectedIndexChanged;
                }

                this.type = value;
                if (this.type != null)
                {
                    this.type.SelectedIndexChanged += this.Type_SelectedIndexChanged;
                }
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// The do correct radio button list.
        /// </summary>
        private void DoCorrectRadioButtonList()
        {
            string _urltype = Convert.ToString(this.ViewState["UrlType"]);

            // Dim objUrls As New UrlController
            if (this.Type.Items.Count <= 0)
            {
                return;
            }

            this.Type.ClearSelection();
            if (!string.IsNullOrEmpty(_urltype))
            {
                if (this.Type.Items.FindByValue(_urltype) != null)
                {
                    this.Type.Items.FindByValue(_urltype).Selected = true;
                }
                else
                {
                    this.Type.Items[0].Selected = true;
                    _urltype = this.Type.Items[0].Value;
                    this.ViewState["UrlType"] = _urltype;
                }
            }
            else
            {
                this.Type.Items[0].Selected = true;
                _urltype = this.Type.Items[0].Value;
                this.ViewState["UrlType"] = _urltype;
            }
        }

        /// <summary>
        /// The do render type controls.
        /// </summary>
        private void DoRenderTypeControls()
        {
            string _url = Convert.ToString(this.ViewState["Url"]);
            string _urltype = Convert.ToString(this.ViewState["UrlType"]);

            if (!string.IsNullOrEmpty(_urltype))
            {
                // load listitems
                switch (this.Type.SelectedItem.Value)
                {
                    case "N":

                        // None
                        this.URLRow.Visible = false;
                        this.FileRow.Visible = false;
                        break;
                    case "U":

                        // Url
                        this.URLRow.Visible = true;
                        this.FileRow.Visible = false;

                        if (string.IsNullOrEmpty(this.UrlTextBox.Text))
                        {
                            this.UrlTextBox.Text = _url;
                        }

                        if (string.IsNullOrEmpty(this.UrlTextBox.Text))
                        {
                            this.UrlTextBox.Text = "http://";
                        }

                        this.UrlTextBox.Visible = true;

                        this.Select.Visible = true;

                        this.Urls.Visible = false;
                        this.Add.Visible = false;
                        this.Delete.Visible = false;
                        break;
                    case "F":

                        // file
                        this.URLRow.Visible = false;
                        this.FileRow.Visible = true;

                        if (this.ViewState["FoldersLoaded"] == null | this._doReloadFolders)
                        {
                            this.LoadFolders("READ,WRITE");
                            this.ViewState["FoldersLoaded"] = "Y";
                        }

                        if (this.Folders.Items.Count == 0)
                        {
                            this.MessageLabel.Text = Localization.GetString("NoPermission", this.LocalResourceFile);
                            this.FileRow.Visible = false;
                            return;
                        }

                        // select folder
                        // We Must check if selected folder has changed because of a property change (Secure, Database)
                        string fileName;
                        string folderPath;
                        string lastFileName = string.Empty;
                        string lastFolderPath = string.Empty;
                        bool _mustRedrawFiles = false;

                        // Let's try to remember last selection
                        if (this.ViewState["LastFolderPath"] != null)
                        {
                            lastFolderPath = Convert.ToString(this.ViewState["LastFolderPath"]);
                        }

                        if (this.ViewState["LastFileName"] != null)
                        {
                            lastFileName = Convert.ToString(this.ViewState["LastFileName"]);
                        }

                        if (_url != string.Empty)
                        {
                            // Let's use the new URL
                            fileName = _url.Substring(_url.LastIndexOf("/", StringComparison.Ordinal) + 1);
                            folderPath = _url.Replace(fileName, string.Empty);
                        }
                        else
                        {
                            // Use last settings
                            fileName = lastFileName;
                            folderPath = lastFolderPath;
                        }

                        if (this.Folders.Items.FindByValue(folderPath) != null)
                        {
                            this.Folders.ClearSelection();
                            this.Folders.Items.FindByValue(folderPath).Selected = true;
                        }
                        else if (this.Folders.Items.Count > 0)
                        {
                            this.Folders.ClearSelection();
                            this.Folders.Items[0].Selected = true;
                            folderPath = this.Folders.Items[0].Value;
                        }

                        if (this.ViewState["FilesLoaded"] == null | folderPath != lastFolderPath | this._doReloadFiles)
                        {
                            // Reload files only if property change or not same folder
                            _mustRedrawFiles = true;
                            this.ViewState["FilesLoaded"] = "Y";
                        }

                        if (_mustRedrawFiles)
                        {
                            this.Files.DataSource = this.GetFileList(true);
                            this.Files.DataBind();
                            if (this.Files.Items.FindByText(fileName) != null)
                            {
                                this.Files.ClearSelection();
                                this.Files.Items.FindByText(fileName).Selected = true;
                            }
                        }

                        this.Files.Visible = true;
                        this.File.Visible = false;

                        var folderInfo = new FolderController().GetFolder(this._portalInfo.PortalID, folderPath, false);
                        this.Upload.Visible = this.ShowUpLoad && FolderPermissionController.CanAddFolder(folderInfo);

                        this.SetStorageLocationType();
                        this.UrlTextBox.Visible = false;
                        this.Save.Visible = false;
                        this.Cancel.Visible = false;

                        if (this.Folders.SelectedIndex >= 0)
                        {
                            this.ViewState["LastFolderPath"] = this.Folders.SelectedValue;
                        }
                        else
                        {
                            this.ViewState["LastFolderPath"] = string.Empty;
                        }

                        if (this.Files.SelectedIndex >= 0)
                        {
                            this.ViewState["LastFileName"] = this.Files.SelectedValue;
                        }
                        else
                        {
                            this.ViewState["LastFileName"] = string.Empty;
                        }

                        break;
                }
            }
            else
            {
                this.URLRow.Visible = false;
                this.FileRow.Visible = false;
            }
        }

        /// <summary>
        /// The do render types.
        /// </summary>
        private void DoRenderTypes()
        {
            // We must clear the list to keep the same item order
            string strCurrent = string.Empty;
            if (this.Type.SelectedIndex >= 0)
            {
                strCurrent = this.Type.SelectedItem.Value;

                // Save current selected value
            }

            this.Type.Items.Clear();

            if (this.Type.Items.FindByValue("N") == null)
            {
                this.Type.Items.Add(new ListItem(Localization.GetString("NoneType", this.LocalResourceFile), "N"));
            }

            if (this.ShowUrls)
            {
                if (this.Type.Items.FindByValue("U") == null)
                {
                    this.Type.Items.Add(new ListItem(Localization.GetString("URLType", this.LocalResourceFile), "U"));
                }
            }
            else
            {
                if (this.Type.Items.FindByValue("U") != null)
                {
                    this.Type.Items.Remove(this.Type.Items.FindByValue("U"));
                }
            }

            if (this.ShowFiles)
            {
                if (this.Type.Items.FindByValue("F") == null)
                {
                    this.Type.Items.Add(new ListItem(Localization.GetString("FileType", this.LocalResourceFile), "F"));
                }
            }
            else
            {
                if (this.Type.Items.FindByValue("F") != null)
                {
                    this.Type.Items.Remove(this.Type.Items.FindByValue("F"));
                }
            }

            if (this.Type.Items.Count > 0)
            {
                if (!string.IsNullOrEmpty(strCurrent))
                {
                    if (this.Type.Items.FindByValue(strCurrent) != null)
                    {
                        this.Type.Items.FindByValue(strCurrent).Selected = true;
                    }
                    else
                    {
                        this.Type.Items[0].Selected = true;
                        this._doRenderTypeControls = true;

                        // Type changed, re-draw
                    }
                }
                else
                {
                    this.Type.Items[0].Selected = true;
                    this._doRenderTypeControls = true;

                    // Type changed, re-draw
                }

                this.TypeRow.Visible = this.Type.Items.Count > 1;
            }
            else
            {
                this.TypeRow.Visible = false;
            }
        }

        /// <summary>
        /// The get file list.
        /// </summary>
        /// <param name="noneSpecified">
        /// The none specified.
        /// </param>
        /// <returns>
        /// The <see cref="ArrayList"/>.
        /// </returns>
        private ArrayList GetFileList(bool noneSpecified)
        {
            return DotNetNuke.Common.Globals.GetFileList(
                this._portalInfo.PortalID, this.FileFilter, noneSpecified, this.Folders.SelectedItem.Value, false);
        }

        /// <summary>
        /// The load folders.
        /// </summary>
        /// <param name="permissions">
        /// The permissions.
        /// </param>
        private void LoadFolders(string permissions)
        {
            var portalId = Null.NullInteger;
            this.Folders.Items.Clear();

            if (!this.IsHostMenu)
            {
                portalId = this._portalInfo.PortalID;
            }

            var foldersList = FileSystemUtils.GetFoldersByUser(portalId, this.ShowSecure, true, permissions);

            foreach (ListItem folderItem in from FolderInfo folder in foldersList
                                            select
                                                new ListItem
                                                    {
                                                        Text =
                                                            folder.FolderPath == Null.NullString
                                                                ? Localization.GetString(
                                                                    "Root", this.LocalResourceFile)
                                                                : folder.FolderPath,
                                                        Value = folder.FolderPath
                                                    })
            {
                this.Folders.Items.Add(folderItem);
            }
        }

        /// <summary>
        /// The load URLs.
        /// </summary>
        private void LoadUrls()
        {
            var objUrls = new UrlController();
            this.Urls.Items.Clear();
            this.Urls.DataSource = objUrls.GetUrls(this._portalInfo.PortalID);
            this.Urls.DataBind();
        }

        /// <summary>
        /// Handles the Init event of the Page control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void Page_Init(object sender, EventArgs e)
        {
            DotNetNuke.Framework.AJAX.RegisterPostBackControl(this.FindControl("Save"));

            // prevent unauthorized access
            if (!this.Request.IsAuthenticated)
            {
                this.Visible = false;
            }
        }

        /// <summary>
        /// Handles the Load event of the Page control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void Page_Load(object sender, EventArgs e)
        {
            try
            {
                var portalController = new PortalController();

                this._portalInfo = portalController.GetPortal(this.PortalSettings.PortalId);

                if (this.ViewState["IsUrlControlLoaded"] != null)
                {
                    return;
                }

                // If Not Page.IsPostBack Then
                // let's make at least an initialization
                // The type radio button must be initialized
                // The url must be initialized no matter its value
                this._doRenderTypes = true;
                this._doChangeURL = true;

                ClientAPI.AddButtonConfirm(this.Delete, Localization.GetString("DeleteItem"));
            }
            catch (Exception exc)
            {
                Exceptions.ProcessModuleLoadException(this, exc);
            }
        }

        /// <summary>
        /// Handles the PreRender event of the Page control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void Page_PreRender(object sender, EventArgs e)
        {
            try
            {
                if (this._doRenderTypes)
                {
                    this.DoRenderTypes();
                }

                if (this._doChangeURL)
                {
                    this.ChangeURL();
                }

                if (this._doReloadFolders | this._doReloadFiles)
                {
                    this.DoCorrectRadioButtonList();
                    this._doRenderTypeControls = true;
                }

                if (this._doRenderTypeControls)
                {
                    if (!(this._doReloadFolders | this._doReloadFiles))
                    {
                        this.DoCorrectRadioButtonList();
                    }

                    this.DoRenderTypeControls();
                }

                this.ViewState["Url"] = null;
                this.ViewState["IsUrlControlLoaded"] = "Loaded";
            }
            catch (Exception exc)
            {
                // Let's detect possible problems
                Exceptions.LogException(new Exception("Error rendering URLControl subcontrols.", exc));
            }
        }

        /// <summary>
        /// The set storage location type.
        /// </summary>
        private void SetStorageLocationType()
        {
            FolderController objFolder = new FolderController();
            string folderName = this.Folders.SelectedValue;

            // Check to see if this is the 'Root' folder, if so we cannot rely on its text value because it is something and not an empty string that we need to lookup the 'root' folder
            if (this.Folders.SelectedValue == string.Empty)
            {
                folderName = string.Empty;
            }

            FolderInfo objFolderInfo = objFolder.GetFolder(this.PortalSettings.PortalId, folderName, false);

            if (objFolderInfo == null)
            {
                return;
            }

            switch ((FolderController.StorageLocationTypes)objFolderInfo.StorageLocation)
            {
                case FolderController.StorageLocationTypes.InsecureFileSystem:
                    this.StorageLocationTypeImage.Visible = false;
                    break;
                case FolderController.StorageLocationTypes.SecureFileSystem:
                    this.StorageLocationTypeImage.ImageUrl = this.ResolveUrl("~/images/icon_securityroles_16px.gif");
                    this.StorageLocationTypeImage.Visible = true;
                    break;
                case FolderController.StorageLocationTypes.DatabaseSecure:
                    this.StorageLocationTypeImage.ImageUrl = this.ResolveUrl("~/images/icon_sql_16px.gif");
                    this.StorageLocationTypeImage.Visible = true;
                    break;
            }
        }

        /// <summary>
        /// Handles the SelectedIndexChanged event of the Folders control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void Folders_SelectedIndexChanged(object sender, EventArgs e)
        {
            var folderInfo = new FolderController().GetFolder(this._portalInfo.PortalID, this.Folders.SelectedValue, false);

            if (FolderPermissionController.CanAddFolder(folderInfo))
            {
                if (!this.File.Visible)
                {
                    this.Save.Visible = false;

                    // only show if not already in upload mode and not disabled
                    this.Upload.Visible = this.ShowUpLoad;
                }
            }
            else
            {
                // reset controls
                this.Files.Visible = true;
                this.Upload.Visible = false;
                this.File.Visible = false;
                this.Save.Visible = false;
                this.Cancel.Visible = false;
            }

            this.Files.Items.Clear();
            this.Files.DataSource = this.GetFileList(true);
            this.Files.DataBind();

            this.SetStorageLocationType();

            if (this.Folders.SelectedIndex >= 0)
            {
                this.ViewState["LastFolderPath"] = this.Folders.SelectedValue;
            }
            else
            {
                this.ViewState["LastFolderPath"] = string.Empty;
            }

            if (this.Files.SelectedIndex >= 0)
            {
                this.ViewState["LastFileName"] = this.Files.SelectedValue;
            }
            else
            {
                this.ViewState["LastFileName"] = string.Empty;
            }

            this._doRenderTypeControls = false;

            // Must not render on this postback
            this._doRenderTypes = false;
            this._doChangeURL = false;
            this._doReloadFolders = false;
            this._doReloadFiles = false;
        }

        /// <summary>
        /// Handles the Click event of the Add control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void Add_Click(object sender, EventArgs e)
        {
            this.Urls.Visible = false;
            this.Select.Visible = true;
            this.UrlTextBox.Visible = true;
            this.Add.Visible = false;
            this.Delete.Visible = false;

            this._doRenderTypeControls = false;

            // Must not render on this postback
            this._doRenderTypes = false;
            this._doChangeURL = false;
            this._doReloadFolders = false;
            this._doReloadFiles = false;
        }

        /// <summary>
        /// Handles the Click event of the Cancel control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void Cancel_Click(object sender, EventArgs e)
        {
            this.Files.Visible = true;
            this.Upload.Visible = true;
            this.File.Visible = false;
            this.Save.Visible = false;
            this.Cancel.Visible = false;

            this._doRenderTypeControls = false;

            // Must not render on this postback
            this._doRenderTypes = false;
            this._doChangeURL = false;
            this._doReloadFolders = false;
            this._doReloadFiles = false;
        }

        /// <summary>
        /// Handles the Click event of the Delete control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void Delete_Click(object sender, EventArgs e)
        {
            if (this.Urls.SelectedItem != null)
            {
                UrlController objUrls = new UrlController();
                objUrls.DeleteUrl(this._portalInfo.PortalID, this.Urls.SelectedItem.Value);

                this.LoadUrls();

                // we must reload the url list
            }

            this._doRenderTypeControls = false;

            // Must not render on this postback
            this._doRenderTypes = false;
            this._doChangeURL = false;
            this._doReloadFolders = false;
            this._doReloadFiles = false;
        }

        /// <summary>
        /// Handles the Click event of the Save control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void Save_Click(object sender, EventArgs e)
        {
            this.Upload.Visible = false;

            // if no file is selected exit
            if (string.IsNullOrEmpty(this.File.PostedFile.FileName))
            {
                return;
            }

            string parentFolderName = this.PortalSettings.ActiveTab.ParentId == this.PortalSettings.SuperTabId
                                          ? DotNetNuke.Common.Globals.HostMapPath
                                          : this.PortalSettings.HomeDirectoryMapPath;

            parentFolderName += this.Folders.SelectedItem.Value;

            string strExtension = Strings.Replace(Path.GetExtension(this.File.PostedFile.FileName), ".", string.Empty);
            if (!string.IsNullOrEmpty(this.FileFilter)
                & Strings.InStr("," + this.FileFilter.ToLower(), "," + strExtension.ToLower()) == 0)
            {
                // trying to upload a file not allowed for current filter
                this.MessageLabel.Text = string.Format(
                    Localization.GetString("UploadError", this.LocalResourceFile), this.FileFilter, strExtension);
            }
            else
            {
                this.MessageLabel.Text = FileSystemUtils.UploadFile(
                    parentFolderName.Replace("/", "\\"), this.File.PostedFile);
            }

            if (this.MessageLabel.Text == string.Empty)
            {
                this.Files.Visible = true;
                this.Upload.Visible = this.ShowUpLoad;
                this.File.Visible = false;
                this.Save.Visible = false;
                this.Cancel.Visible = false;

                this.Files.Items.Clear();
                this.Files.DataSource = this.GetFileList(false);
                this.Files.DataBind();

                string fileName =
                    this.File.PostedFile.FileName.Substring(
                        this.File.PostedFile.FileName.LastIndexOf("\\", StringComparison.Ordinal) + 1);
                if (this.Files.Items.FindByText(fileName) != null)
                {
                    this.Files.Items.FindByText(fileName).Selected = true;
                }

                if (this.Files.SelectedIndex >= 0)
                {
                    this.ViewState["LastFileName"] = this.Files.SelectedValue;
                }
                else
                {
                    this.ViewState["LastFileName"] = string.Empty;
                }
            }

            this._doRenderTypeControls = false;

            // Must not render on this postback
            this._doRenderTypes = false;
            this._doChangeURL = false;
            this._doReloadFolders = false;
            this._doReloadFiles = false;
        }

        /// <summary>
        /// Handles the Click event of the Select control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void Select_Click(object sender, EventArgs e)
        {
            this.Urls.Visible = true;
            this.Select.Visible = false;
            this.UrlTextBox.Visible = false;
            this.Add.Visible = true;
            this.Delete.Visible = PortalSecurity.IsInRole(this._portalInfo.AdministratorRoleName);

            this.LoadUrls();

            if (this.Urls.Items.FindByValue(this.UrlTextBox.Text) != null)
            {
                this.Urls.ClearSelection();
                this.Urls.Items.FindByValue(this.UrlTextBox.Text).Selected = true;
            }

            this._doRenderTypeControls = false;

            // Must not render on this postback
            this._doRenderTypes = false;
            this._doChangeURL = false;
            this._doReloadFolders = false;
            this._doReloadFiles = false;
        }

        /// <summary>
        /// Handles the Click event of the Upload control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void Upload_Click(object sender, EventArgs e)
        {
            string strSaveFolder = this.Folders.SelectedValue;
            this.LoadFolders("WRITE");
            if (this.Folders.Items.FindByValue(strSaveFolder) != null)
            {
                this.Folders.Items.FindByValue(strSaveFolder).Selected = true;
                this.Files.Visible = false;
                this.Upload.Visible = false;
                this.File.Visible = true;
                this.Save.Visible = true;
                this.Cancel.Visible = true;
            }
            else
            {
                if (this.Folders.Items.Count > 0)
                {
                    this.Folders.Items[0].Selected = true;
                    this.Files.Visible = false;
                    this.Upload.Visible = false;
                    this.File.Visible = true;
                    this.Save.Visible = true;
                    this.Cancel.Visible = true;
                }
                else
                {
                    // reset controls
                    this.LoadFolders("READ,WRITE");
                    this.Folders.Items.FindByValue(strSaveFolder).Selected = true;
                    this.Files.Visible = true;
                    this.Upload.Visible = false;
                    this.File.Visible = false;
                    this.Save.Visible = false;
                    this.Cancel.Visible = false;
                    this.MessageLabel.Text = Localization.GetString("NoWritePermission", this.LocalResourceFile);
                }
            }

            this._doRenderTypeControls = false;

            // Must not render on this postback
            this._doRenderTypes = false;
            this._doChangeURL = false;
            this._doReloadFolders = false;
            this._doReloadFiles = false;
        }

        /// <summary>
        /// Changes the URL.
        /// </summary>
        private void ChangeURL()
        {
            string _url = Convert.ToString(this.ViewState["Url"]);
            string _urltype = Convert.ToString(this.ViewState["UrlType"]);
            if (!string.IsNullOrEmpty(_url))
            {
                _urltype = DotNetNuke.Common.Globals.GetURLType(_url).ToString("g").Substring(0, 1);
                this.ViewState["UrlType"] = _urltype;

                if (_urltype == "F")
                {
                    FileController objFiles = new FileController();
                    if (_url.ToLower().StartsWith("fileid="))
                    {
                        FileInfo objFile = objFiles.GetFileById(int.Parse(_url.Substring(7)), this._portalInfo.PortalID);
                        if (objFile != null)
                        {
                            _url = objFile.Folder + objFile.FileName;
                        }
                    }
                }

                this.ViewState["Url"] = _url;
            }
            else
            {
                if (!string.IsNullOrEmpty(_urltype))
                {
                    this.Type.ClearSelection();
                    if (this.Type.Items.FindByValue(_urltype) != null)
                    {
                        this.Type.Items.FindByValue(_urltype).Selected = true;
                    }
                    else
                    {
                        this.Type.Items[0].Selected = true;
                    }
                }
                else
                {
                    if (this.Type.Items.Count > 0)
                    {
                        this.Type.ClearSelection();
                        this.Type.Items[0].Selected = true;
                    }
                }
            }

            // Url type changed, then we must draw the controlos for that type
            this._doRenderTypeControls = true;
        }

        /// <summary>
        /// Type changed, render the correct control set
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void Type_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.ViewState["UrlType"] = this.Type.SelectedItem.Value;
            this._doRenderTypeControls = true;
        }

        #endregion
    }
}